Package: ggtext
Type: Package
Title: Improved Text Rendering Support for 'ggplot2'
Version: 0.2.0
Authors@R: 
    c(
      person(
        given = "Claus O.",
        family = "Wilke",
        role = c("aut"),
        email = "wilke@austin.utexas.edu",
        comment = c(ORCID = "0000-0002-7470-9261")
      ),
      person(
        given = "Brenton M.",
        family = "Wiernik",
        role = c("aut", "cre"),
        email = "brenton@wiernik.org",
        comment = c(ORCID = "0000-0001-9560-6336", Twitter = "@bmwiernik")
      )
    )
Description: A 'ggplot2' extension that enables the rendering of
    complex formatted plot labels (titles, subtitles, facet labels,
    axis labels, etc.). Text boxes with automatic word wrap are also
    supported.
URL: https://wilkelab.org/ggtext/
BugReports: https://github.com/wilkelab/ggtext/issues
License: GPL-2
Depends: R (>= 3.5)
Imports: ggplot2 (>= 3.3.0), grid, gridtext, rlang, scales
Suggests: cowplot, dplyr, glue, knitr, rmarkdown, testthat, vdiffr
Encoding: UTF-8
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-26 18:23:23 UTC; brentonw
Author: Claus O. Wilke [aut] (ORCID: <https://orcid.org/0000-0002-7470-9261>),
  Brenton M. Wiernik [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9560-6336>, Twitter: @bmwiernik)
Maintainer: Brenton M. Wiernik <brenton@wiernik.org>
Repository: CRAN
Date/Publication: 2026-08-28 10:30:07 UTC
Built: R 4.5.2; ; 2026-08-28 14:16:02 UTC; unix

Package: ggtext
Type: Package
Title: Improved Text Rendering Support for 'ggplot2'
Version: 0.1.2
Authors@R: 
    c(
      person(
        given = "Claus O.",
        family = "Wilke",
        role = c("aut"),
        email = "wilke@austin.utexas.edu",
        comment = c(ORCID = "0000-0002-7470-9261")
      ),
      person(
        given = "Brenton M.",
        family = "Wiernik",
        role = c("aut", "cre"),
        email = "brenton@wiernik.org",
        comment = c(ORCID = "0000-0001-9560-6336", Twitter = "@bmwiernik")
      )
    )
Description: A 'ggplot2' extension that enables the rendering of
    complex formatted plot labels (titles, subtitles, facet labels,
    axis labels, etc.). Text boxes with automatic word wrap are also
    supported.
URL: https://wilkelab.org/ggtext/
BugReports: https://github.com/wilkelab/ggtext/issues
License: GPL-2
Depends: R (>= 3.5)
Imports: ggplot2 (>= 3.3.0), grid, gridtext, rlang, scales
Suggests: cowplot, dplyr, glue, knitr, rmarkdown, testthat, vdiffr
Encoding: UTF-8
RoxygenNote: 7.1.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2022-09-15 13:50:50 UTC; brentonw
Author: Claus O. Wilke [aut] (<https://orcid.org/0000-0002-7470-9261>),
  Brenton M. Wiernik [aut, cre] (<https://orcid.org/0000-0001-9560-6336>,
    @bmwiernik)
Maintainer: Brenton M. Wiernik <brenton@wiernik.org>
Repository: CRAN
Date/Publication: 2022-09-16 11:36:07 UTC
Built: R 4.6.0; ; 2026-04-28 13:38:14 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(element_grob,element_markdown)
S3method(element_grob,element_textbox)
export(GeomRichText)
export(GeomRichtext)
export(GeomTextBox)
export(draw_key_richtext)
export(element_markdown)
export(element_textbox)
export(element_textbox_simple)
export(geom_richtext)
export(geom_textbox)
import(ggplot2)
import(grid)
import(gridtext)
import(rlang)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ggtext: Improved text rendering support for ggplot2

<!-- badges: start -->

[![R build
status](https://github.com/wilkelab/ggtext/workflows/R-CMD-check/badge.svg)](https://github.com/wilkelab/ggtext/actions)
[![Coverage
Status](https://img.shields.io/codecov/c/github/wilkelab/ggtext/master.svg)](https://codecov.io/github/wilkelab/ggtext?branch=master)
[![CRAN
status](https://www.r-pkg.org/badges/version/ggtext)](https://cran.r-project.org/package=ggtext)
[![Lifecycle:
maturing](https://img.shields.io/badge/lifecycle-maturing-blue.svg)](https://lifecycle.r-lib.org/articles/stages.html#maturing)
<!-- badges: end -->

The ggtext package provides simple Markdown and HTML rendering for
ggplot2. Under the hood, the package uses the
[gridtext](https://CRAN.R-project.org/package=gridtext) package for the
actual rendering, and consequently it is limited to the feature set
provided by gridtext.

Support is provided for Markdown both in theme elements (plot titles,
subtitles, captions, axis labels, legends, etc.) and in geoms (similar
to `geom_text()`). In both cases, there are two alternatives, one for
creating simple text labels and one for creating text boxes with word
wrapping.

Importantly, the gridtext package that provides the rendering support
**implements only an extremely limited subset of Markdown/HTML/CSS.** It
currently can make text bold or italics, can change the font, color, or
size of a piece of text, can place text as sub- or superscript, and has
extremely rudimentary image support. No other features are currently
supported. As a general rule, any Markdown, HTML, or CSS feature that
isn’t shown in any of the ggtext or gridtext documentation likely
doesn’t exist.

## Installation

You can install the latest stable release from CRAN via
`install.packages()`:

``` r
install.packages("ggtext")
```

To install the latest development version of this package, please run
the following line in your R console:

``` r
remotes::install_github("wilkelab/ggtext")
```

## Markdown in theme elements

The ggtext package defines two new theme elements, `element_markdown()`
and `element_textbox()`. Both behave similarly to `element_text()` but
render the provided text as markdown/html. `element_markdown()` is meant
as a direct replacement for `element_text()`, and it renders text
without word wrapping. To start a new line, use the `<br>` tag or add
two spaces before the end of a line.

As an example, we can mix regular, italics, and bold text, and we can
also apply colors to axis tick labels. This particular example was
inspired by [this stackoverflow
post.](https://stackoverflow.com/questions/39282293/r-ggplot2-using-italics-and-non-italics-in-the-same-category-label)

``` r
library(tidyverse)
library(ggtext)
library(glue)

data <- tibble(
  bactname = c("Staphylococcaceae", "Moraxella", "Streptococcus", "Acinetobacter"),
  OTUname = c("OTU 1", "OTU 2", "OTU 3", "OTU 4"),
  value = c(-0.5, 0.5, 2, 3)
)

data %>% mutate(
  color = c("#009E73", "#D55E00", "#0072B2", "#000000"),
  name = glue("<i style='color:{color}'>{bactname}</i> ({OTUname})"),
  name = fct_reorder(name, value)
)  %>%
  ggplot(aes(value, name, fill = color)) + 
  geom_col(alpha = 0.5) + 
  scale_fill_identity() +
  labs(caption = "Example posted on **stackoverflow.com**<br>(using made-up data)") +
  theme(
    axis.text.y = element_markdown(),
    plot.caption = element_markdown(lineheight = 1.2)
  )
```

![](man/figures/README-unnamed-chunk-4-1.png)<!-- -->

Very basic support for the `<img>` tag exists, and it can be used, for
example, to employ images as axis labels.

``` r
labels <- c(
  setosa = "<img src='https://upload.wikimedia.org/wikipedia/commons/thumb/8/86/Iris_setosa.JPG/180px-Iris_setosa.JPG'
    width='100' /><br>*I. setosa*",
  virginica = "<img src='https://upload.wikimedia.org/wikipedia/commons/thumb/3/38/Iris_virginica_-_NRCS.jpg/320px-Iris_virginica_-_NRCS.jpg'
    width='100' /><br>*I. virginica*",
  versicolor = "<img src='https://upload.wikimedia.org/wikipedia/commons/thumb/2/27/20140427Iris_versicolor1.jpg/320px-20140427Iris_versicolor1.jpg'
    width='100' /><br>*I. versicolor*"
)

ggplot(iris, aes(Species, Sepal.Width)) +
  geom_boxplot() +
  scale_x_discrete(
    name = NULL,
    labels = labels
  ) +
  theme(
    axis.text.x = element_markdown(color = "black", size = 11)
  )
```

![](man/figures/README-unnamed-chunk-5-1.png)<!-- -->

`element_textbox()` offers support for rendering larger amounts of text
that require word wrapping. Unlike `element_markdown()`, it cannot be
used for axis tick labels, and it cannot draw text at arbitrary angles,
only at fixed orientations corresponding to 0, 90, 180, and 270 degrees.
In practice, you will usually want to use `element_textbox_simple()`
instead of `element_textbox()`, as it sets useful defaults for many
parameters not usually defined in ggplot2 themes.

``` r
ggplot(mtcars, aes(disp, mpg)) + 
  geom_point() +
  labs(
    title = "<b>Fuel economy vs. engine displacement</b><br>
    <span style = 'font-size:10pt'>Lorem ipsum *dolor sit amet,*
    consectetur adipiscing elit, **sed do eiusmod tempor incididunt** ut
    labore et dolore magna aliqua. <span style = 'color:red;'>Ut enim
    ad minim veniam,</span> quis nostrud exercitation ullamco laboris nisi
    ut aliquip ex ea commodo consequat.</span>",
    x = "displacement (in<sup>3</sup>)",
    y = "Miles per gallon (mpg)<br><span style = 'font-size:8pt'>A measure of
    the car's fuel efficiency.</span>"
  ) +
  theme(
    plot.title.position = "plot",
    plot.title = element_textbox_simple(
      size = 13,
      lineheight = 1,
      padding = margin(5.5, 5.5, 5.5, 5.5),
      margin = margin(0, 0, 5.5, 0),
      fill = "cornsilk"
    ),
    axis.title.x = element_textbox_simple(
      width = NULL,
      padding = margin(4, 4, 4, 4),
      margin = margin(4, 0, 0, 0),
      linetype = 1,
      r = grid::unit(8, "pt"),
      fill = "azure1"
    ),
    axis.title.y = element_textbox_simple(
      hjust = 0,
      orientation = "left-rotated",
      minwidth = unit(1, "in"),
      maxwidth = unit(2, "in"),
      padding = margin(4, 4, 2, 4),
      margin = margin(0, 0, 2, 0),
      fill = "lightsteelblue1"
    )
  )
```

![](man/figures/README-unnamed-chunk-6-1.png)<!-- -->

Another example, replacing facet strips with text boxes.

``` r
library(cowplot)

ggplot(mpg, aes(cty, hwy)) + 
  geom_point() +
  facet_wrap(~class) +
  theme_half_open(12) +
  background_grid() +
  theme(
    strip.background = element_blank(),
    strip.text = element_textbox(
      size = 12,
      color = "white", fill = "#5D729D", box.color = "#4A618C",
      halign = 0.5, linetype = 1, r = unit(5, "pt"), width = unit(1, "npc"),
      padding = margin(2, 0, 1, 0), margin = margin(3, 3, 3, 3)
    )
  )
#> Warning: The `size` argument of `element_line()` is deprecated as of ggplot2 3.4.0.
#> Please use the `linewidth` argument instead.
#> Warning: The `size` argument of `element_rect()` is deprecated as of ggplot2 3.4.0.
#> Please use the `linewidth` argument instead.
```

![](man/figures/README-unnamed-chunk-7-1.png)<!-- -->

## Geoms

The geom `geom_richtext()` provides markdown/html labels. Unlike
`geom_label()`, the labels can be rotated.

``` r
df <- tibble(
  label = c(
    "Some text **in bold.**",
    "Linebreaks<br>Linebreaks<br>Linebreaks",
    "*x*<sup>2</sup> + 5*x* + *C*<sub>*i*</sub>",
    "Some <span style='color:blue'>blue text **in bold.**</span><br>And *italics text.*<br>
    And some <span style='font-size:18pt; color:black'>large</span> text."
  ),
  x = c(.2, .1, .5, .9),
  y = c(.8, .4, .1, .5),
  hjust = c(0.5, 0, 0, 1),
  vjust = c(0.5, 1, 0, 0.5),
  angle = c(0, 0, 45, -45),
  color = c("black", "blue", "black", "red"),
  fill = c("cornsilk", "white", "lightblue1", "white")
)


ggplot(df) +
  aes(
    x, y, label = label, angle = angle, color = color, fill = fill,
    hjust = hjust, vjust = vjust
  ) +
  geom_richtext() +
  geom_point(color = "black", size = 2) +
  scale_color_identity() +
  scale_fill_identity() +
  xlim(0, 1) + ylim(0, 1)
```

![](man/figures/README-unnamed-chunk-8-1.png)<!-- -->

Labels without frame or background are also possible.

``` r
ggplot(df) +
  aes(
    x, y, label = label, angle = angle, color = color,
    hjust = hjust, vjust = vjust
  ) +
  geom_richtext(
    fill = NA, label.color = NA, # remove background and outline
    label.padding = grid::unit(rep(0, 4), "pt") # remove padding
  ) +
  geom_point(color = "black", size = 2) +
  scale_color_identity() +
  xlim(0, 1) + ylim(0, 1)
```

![](man/figures/README-unnamed-chunk-9-1.png)<!-- -->

The geom `geom_textbox()` can draw boxes with word-wrapped text. It does
not support arbitrary rotation angles, only fixed orientations, just
like `element_textbox()`.

``` r
df <- tibble(
  label = rep("Lorem ipsum dolor **sit amet,** consectetur adipiscing elit,
    sed do *eiusmod tempor incididunt* ut labore et dolore magna
    aliqua.", 2),
  x = c(0, .6),
  y = c(1, .6),
  hjust = c(0, 0),
  vjust = c(1, 0),
  orientation = c("upright", "right-rotated"),
  color = c("black", "blue"),
  fill = c("cornsilk", "white")
)

ggplot(df) +
  aes(
    x, y, label = label, color = color, fill = fill,
    hjust = hjust, vjust = vjust,
    orientation = orientation
  ) +
  geom_textbox(width = unit(0.4, "npc")) +
  geom_point(color = "black", size = 2) +
  scale_discrete_identity(aesthetics = c("color", "fill", "orientation")) +
  xlim(0, 1) + ylim(0, 1)
```

![](man/figures/README-unnamed-chunk-10-1.png)<!-- -->

## Acknowledgments

This project received [financial
support](https://www.r-consortium.org/all-projects/awarded-projects)
from the [R consortium.](https://www.r-consortium.org)

<img src="https://www.r-consortium.org/wp-content/uploads/sites/13/2016/09/RConsortium_Horizontal_Pantone.png" width="250">

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 4,
  fig.height = 3.75
)

## ----fig.width = 6, fig.height = 3, message = FALSE---------------------------
library(ggplot2)
library(dplyr)
library(glue)

iris_cor <- iris %>% 
  group_by(Species) %>%
  summarize(r_square = cor(Sepal.Length, Sepal.Width)^2) %>%
  mutate(
    # location of each text label in data coordinates
    Sepal.Length = 8, Sepal.Width = 4.5,
    # text label containing r^2 value 
    label = glue("r^2 = {round(r_square, 2)}")
  )

iris_cor

iris_facets <- ggplot(iris, aes(Sepal.Length, Sepal.Width)) +
  geom_point() + 
  geom_smooth(method = "lm", formula = y ~ x) +
  facet_wrap(~Species) +
  theme_bw()

iris_facets + 
  geom_text(
    data = iris_cor,
    aes(label = label),
    hjust = 1, vjust = 1
  )

## ----fig.width = 6, fig.height = 3, message = FALSE---------------------------
library(ggtext)

iris_cor_md <- iris_cor %>% 
  mutate(
    # markdown version of text label
    label = glue("*r*<sup>2</sup> = {round(r_square, 2)}")
  )

iris_cor_md

iris_facets + 
  geom_richtext(
    data = iris_cor_md,
    aes(label = label),
    hjust = 1, vjust = 1
  )

## ----fig.width = 6, fig.height = 3--------------------------------------------
iris_facets + 
  geom_richtext(
    data = iris_cor_md,
    aes(label = label),
    hjust = 1, vjust = 1,
    # remove label background and outline
    fill = NA, label.color = NA,
    # remove label padding, since we have removed the label outline
    label.padding = grid::unit(rep(0, 4), "pt") 
  )

## ----fig.width = 6, fig.height = 3--------------------------------------------
iris_facets + 
  aes(colour = Species) +
  geom_richtext(
    data = iris_cor_md,
    aes(
      label = label,
      fill = after_scale(alpha(colour, .2))
    ),
    text.colour = "black",
    hjust = 1, vjust = 1
  ) +
  theme(legend.position = "none")

## ----fig.width = 6, fig.height = 3--------------------------------------------
iris_facets + 
  aes(colour = Species) +
  geom_richtext(
    data = iris_cor_md,
    aes(
      x = 7.5,
      label = label,
      fill = after_scale(alpha(colour, .2))
    ),
    text.colour = "black",
    hjust = 1, vjust = 1,
    angle = 30
  ) +
  theme(legend.position = "none")

## ----fig.width = 6, fig.height = 3--------------------------------------------
df <- data.frame(
  x = 0.1,
  y = 0.8,
  label = "*Lorem ipsum dolor sit amet,* consectetur adipiscing
elit. Quisque tincidunt eget arcu in pulvinar. Morbi varius leo
vel consectetur luctus. **Morbi facilisis justo non fringilla.**
Vivamus sagittis sem felis, vel lobortis risus mattis eget. Nam
quis imperdiet felis, in convallis elit."
)

p <- ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label),
    width = grid::unit(0.73, "npc"), # 73% of plot panel width
    hjust = 0, vjust = 1
  ) +
  xlim(0, 1) + ylim(0, 1)

p

## ----fig.width = 4, fig.height = 4--------------------------------------------
p

## ----fig.width = 4, fig.height = 4--------------------------------------------
ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label),
    width = grid::unit(0.73, "npc"), # 73% of plot panel width
    hjust = 0, vjust = 1,
    halign = 0.5 # centered text
  ) +
  xlim(0, 1) + ylim(0, 1)

## -----------------------------------------------------------------------------
df <- data.frame(
  x = 0.5,
  y = 0.5,
  label = "The quick brown fox jumps over the lazy dog.",
  orientation = c("upright", "left-rotated", "inverted", "right-rotated")
)

ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label, orientation = orientation),
    width = grid::unit(1.5, "in"),
    height = grid::unit(1.5, "in"),
    box.margin = grid::unit(rep(0.25, 4), "in"),
    hjust = 0, vjust = 1
  ) +
  xlim(0, 1) + ylim(0, 1) +
  scale_discrete_identity(aesthetics = "orientation")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 4,
  fig.height = 3.75
)

## ----fig.width = 6, fig.height = 3, message = FALSE---------------------------
library(ggplot2)
library(dplyr)
library(glue)

iris_cor <- iris %>% 
  group_by(Species) %>%
  summarize(r_square = cor(Sepal.Length, Sepal.Width)^2) %>%
  mutate(
    # location of each text label in data coordinates
    Sepal.Length = 8, Sepal.Width = 4.5,
    # text label containing r^2 value 
    label = glue("r^2 = {round(r_square, 2)}")
  )

iris_cor

iris_facets <- ggplot(iris, aes(Sepal.Length, Sepal.Width)) +
  geom_point() + 
  geom_smooth(method = "lm", formula = y ~ x) +
  facet_wrap(~Species) +
  theme_bw()

iris_facets + 
  geom_text(
    data = iris_cor,
    aes(label = label),
    hjust = 1, vjust = 1
  )

## ----fig.width = 6, fig.height = 3, message = FALSE---------------------------
library(ggtext)

iris_cor_md <- iris_cor %>% 
  mutate(
    # markdown version of text label
    label = glue("*r*<sup>2</sup> = {round(r_square, 2)}")
  )

iris_cor_md

iris_facets + 
  geom_richtext(
    data = iris_cor_md,
    aes(label = label),
    hjust = 1, vjust = 1
  )

## ----fig.width = 6, fig.height = 3--------------------------------------------
iris_facets + 
  geom_richtext(
    data = iris_cor_md,
    aes(label = label),
    hjust = 1, vjust = 1,
    # remove label background and outline
    fill = NA, label.color = NA,
    # remove label padding, since we have removed the label outline
    label.padding = grid::unit(rep(0, 4), "pt") 
  )

## ----fig.width = 6, fig.height = 3--------------------------------------------
iris_facets + 
  aes(colour = Species) +
  geom_richtext(
    data = iris_cor_md,
    aes(
      label = label,
      fill = after_scale(alpha(colour, .2))
    ),
    text.colour = "black",
    hjust = 1, vjust = 1
  ) +
  theme(legend.position = "none")

## ----fig.width = 6, fig.height = 3--------------------------------------------
iris_facets + 
  aes(colour = Species) +
  geom_richtext(
    data = iris_cor_md,
    aes(
      x = 7.5,
      label = label,
      fill = after_scale(alpha(colour, .2))
    ),
    text.colour = "black",
    hjust = 1, vjust = 1,
    angle = 30
  ) +
  theme(legend.position = "none")

## ----fig.width = 6, fig.height = 3--------------------------------------------
df <- data.frame(
  x = 0.1,
  y = 0.8,
  label = "*Lorem ipsum dolor sit amet,* consectetur adipiscing
elit. Quisque tincidunt eget arcu in pulvinar. Morbi varius leo
vel consectetur luctus. **Morbi facilisis justo non fringilla.**
Vivamus sagittis sem felis, vel lobortis risus mattis eget. Nam
quis imperdiet felis, in convallis elit."
)

p <- ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label),
    width = grid::unit(0.73, "npc"), # 73% of plot panel width
    hjust = 0, vjust = 1
  ) +
  xlim(0, 1) + ylim(0, 1)

p

## ----fig.width = 4, fig.height = 4--------------------------------------------
p

## ----fig.width = 4, fig.height = 4--------------------------------------------
ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label),
    width = grid::unit(0.73, "npc"), # 73% of plot panel width
    hjust = 0, vjust = 1,
    halign = 0.5 # centered text
  ) +
  xlim(0, 1) + ylim(0, 1)

## -----------------------------------------------------------------------------
df <- data.frame(
  x = 0.5,
  y = 0.5,
  label = "The quick brown fox jumps over the lazy dog.",
  orientation = c("upright", "left-rotated", "inverted", "right-rotated")
)

ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label, orientation = orientation),
    width = grid::unit(1.5, "in"),
    height = grid::unit(1.5, "in"),
    box.margin = grid::unit(rep(0.25, 4), "in"),
    hjust = 0, vjust = 1
  ) +
  xlim(0, 1) + ylim(0, 1) +
  scale_discrete_identity(aesthetics = "orientation")

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 4,
  fig.height = 3.75
)

## ----message = FALSE----------------------------------------------------------
library(ggplot2)
library(ggtext)

base <- ggplot(data.frame(x = c(-5, 5)), aes(x)) +
  stat_function(fun = ~ .x*.x)

base

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title.x = element_markdown(color = "blue"),
    axis.title.y = element_markdown(size = rel(0.8))
  )

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  scale_y_continuous(position = "right") +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  scale_y_continuous(position = "right") +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y.right = element_markdown()
  )

## ----fig.width = 5, message = FALSE-------------------------------------------
library(dplyr)

mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG)",
    title = "<span style = 'font-size:14pt; font-family:Helvetica;'>Transmission type impacts fuel efficiency</span><br>
MPG is higher for <span style = 'color:#0072B2;'>automatic</span>
than for <span style = 'color:#D55E00;'>manual</span> transmissions"
  ) +
  theme_bw() +
  theme(
    text = element_text(family = "Times"),
    plot.title.position = "plot",
    plot.title = element_markdown(size = 11, lineheight = 1.2)
  )


## ----fig.width = 5, message = FALSE-------------------------------------------
base <- mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG)",
    title = "Transmission type impacts fuel efficiency<br>
<span style = 'font-size:10pt;'>Miles per gallon (MPG) is on average higher for cars
with <span style = 'color:#0072B2;'>automatic transmission</span> than for cars with
<span style = 'color:#D55E00;'>manual transmission.</span> However, MPG generally
declines with increasing horse power.</span>"
  ) +
  theme_bw() + theme(plot.title.position = "plot")

base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1, padding = margin(0, 0, 5, 0)
    )
  )

## ----fig.width = 5, message = FALSE-------------------------------------------
base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1,
      linetype = 1, # turn on border
      box.color = "#748696", # border color
      fill = "#F0F7FF", # background fill color
      r = grid::unit(3, "pt"), # radius for rounded corners
      padding = margin(5, 5, 5, 5), # padding around text inside the box
      margin = margin(0, 0, 10, 0) # margin outside the box
    )
  )

## ----fig.width = 5, message = FALSE-------------------------------------------
base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1, 
      width = grid::unit(4, "in"), # fixed width
      hjust = 1, # alignment of box relative to plot
      linetype = 1, # turn on border
      box.color = "#748696", # border color
      fill = "#F0F7FF", # background fill color
      r = grid::unit(3, "pt"), # radius for rounded corners
      padding = margin(5, 5, 5, 5), # padding around text inside the box
      margin = margin(0, 0, 10, 0) # margin outside the box
    )
  )

## -----------------------------------------------------------------------------
mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG) is on average higher for cars with <span style =
'color:#0072B2;'>automatic transmission</span> than for cars with <span style =
'color:#D55E00;'>manual transmission.</span>",
    title = "Transmission type impacts fuel efficiency"
  ) +
  theme_bw() + 
  theme(
    plot.title.position = "plot",
    axis.title.y = element_textbox_simple(
      orientation = "left-rotated",
      width = grid::unit(2.5, "in"),
      hjust = 0, fill = "#F0F7FF",  
      padding = margin(5, 5, 5, 5),
      margin = margin(0, 0, 10, 0)
    )
  )

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 4,
  fig.height = 3.75
)

## ----message = FALSE----------------------------------------------------------
library(ggplot2)
library(ggtext)

base <- ggplot(data.frame(x = c(-5, 5)), aes(x)) +
  stat_function(fun = ~ .x*.x)

base

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title.x = element_markdown(color = "blue"),
    axis.title.y = element_markdown(size = rel(0.8))
  )

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  scale_y_continuous(position = "right") +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )

## -----------------------------------------------------------------------------
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  scale_y_continuous(position = "right") +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y.right = element_markdown()
  )

## ----fig.width = 5, message = FALSE-------------------------------------------
library(dplyr)

mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG)",
    title = "<span style = 'font-size:14pt; font-family:Helvetica;'>Transmission type impacts fuel efficiency</span><br>
MPG is higher for <span style = 'color:#0072B2;'>automatic</span>
than for <span style = 'color:#D55E00;'>manual</span> transmissions"
  ) +
  theme_bw() +
  theme(
    text = element_text(family = "Times"),
    plot.title.position = "plot",
    plot.title = element_markdown(size = 11, lineheight = 1.2)
  )


## ----fig.width = 5, message = FALSE-------------------------------------------
base <- mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG)",
    title = "Transmission type impacts fuel efficiency<br>
<span style = 'font-size:10pt;'>Miles per gallon (MPG) is on average higher for cars
with <span style = 'color:#0072B2;'>automatic transmission</span> than for cars with
<span style = 'color:#D55E00;'>manual transmission.</span> However, MPG generally
declines with increasing horse power.</span>"
  ) +
  theme_bw() + theme(plot.title.position = "plot")

base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1, padding = margin(0, 0, 5, 0)
    )
  )

## ----fig.width = 5, message = FALSE-------------------------------------------
base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1,
      linetype = 1, # turn on border
      box.color = "#748696", # border color
      fill = "#F0F7FF", # background fill color
      r = grid::unit(3, "pt"), # radius for rounded corners
      padding = margin(5, 5, 5, 5), # padding around text inside the box
      margin = margin(0, 0, 10, 0) # margin outside the box
    )
  )

## ----fig.width = 5, message = FALSE-------------------------------------------
base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1, 
      width = grid::unit(4, "in"), # fixed width
      hjust = 1, # alignment of box relative to plot
      linetype = 1, # turn on border
      box.color = "#748696", # border color
      fill = "#F0F7FF", # background fill color
      r = grid::unit(3, "pt"), # radius for rounded corners
      padding = margin(5, 5, 5, 5), # padding around text inside the box
      margin = margin(0, 0, 10, 0) # margin outside the box
    )
  )

## -----------------------------------------------------------------------------
mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG) is on average higher for cars with <span style =
'color:#0072B2;'>automatic transmission</span> than for cars with <span style =
'color:#D55E00;'>manual transmission.</span>",
    title = "Transmission type impacts fuel efficiency"
  ) +
  theme_bw() + 
  theme(
    plot.title.position = "plot",
    axis.title.y = element_textbox_simple(
      orientation = "left-rotated",
      width = grid::unit(2.5, "in"),
      hjust = 0, fill = "#F0F7FF",  
      padding = margin(5, 5, 5, 5),
      margin = margin(0, 0, 10, 0)
    )
  )

#' Improved text rendering support for ggplot2
#'
#' The ggtext package implements both geoms ([geom_richtext()], [geom_textbox()])
#' and theme elements ([element_markdown()], [element_textbox()]) for improved
#' text rendering with ggplot2.
#'
#' @name ggtext
#' @docType package
#' @import grid
#' @import rlang
#' @import gridtext
#' @import ggplot2
NULL

#' Richtext labels
#'
#' This geom draws text labels similar to [ggplot2::geom_label()], but formatted
#' using basic markdown/html. Parameter and aesthetic names follow the conventions
#' of [ggplot2::geom_label()], and therefore the appearance of the frame around
#' the label is controlled with `label.colour`, `label.padding`, `label.margin`,
#' `label.size`, `label.r`, even though the same parameters are called `box.colour`,
#' `box.padding`, `box.margin`, `box.size`, and `box.r` in [geom_textbox()]. Most 
#' styling parameters can be used as aesthetics and can be applied separately to
#' each text label drawn. The exception is styling parameters that are specified
#' as grid units (e.g., `label.padding` or `label.r`), which can only be specified
#' for all text labels at once. See examples for details.
#' 
#' @section Aesthetics:
#' 
#' `geom_richtext()` understands the following aesthetics (required 
#' aesthetics are in bold; select aesthetics are annotated):
#' 
#' * **`x`**
#' * **`y`**
#' * **`label`**
#' * `alpha`
#' * `angle`
#' * `colour` Default color of label text and label outline.
#' * `family`
#' * `fontface`
#' * `fill` Default fill color of label background.
#' * `group`
#' * `hjust`
#' * `label.colour` Color of label outline. Overrides `colour`.
#' * `label.size` Width of label outline.
#' * `lineheight`
#' * `size` Default font size of label text.
#' * `text.colour` Color of label text. Overrides `colour`. 
#' * `vjust`
#' 
#' @inheritParams ggplot2::geom_text
#' @inheritParams ggplot2::geom_label
#' @param label.margin Unit vector of length four specifying the margin
#'   outside the text label.
#' @return A ggplot2 layer that can be added to a plot created with
#'   [ggplot2::ggplot()].
#' @seealso [geom_textbox()], [element_markdown()]
#' @examples
#' library(ggplot2)
#' 
#' df <- data.frame(
#'   label = c(
#'     "Some text **in bold.**",
#'     "Linebreaks<br>Linebreaks<br>Linebreaks",
#'     "*x*<sup>2</sup> + 5*x* + *C*<sub>*i*</sub>",
#'     "Some <span style='color:blue'>blue text **in bold.**</span><br>And *italics text.*<br>
#'       And some <span style='font-size:18pt; color:black'>large</span> text."
#'   ),
#'   x = c(.2, .1, .5, .9),
#'   y = c(.8, .4, .1, .5),
#'   hjust = c(0.5, 0, 0, 1),
#'   vjust = c(0.5, 1, 0, 0.5),
#'   angle = c(0, 0, 45, -45),
#'   color = c("black", "blue", "black", "red"),
#'   fill = c("cornsilk", "white", "lightblue1", "white")
#' )
#' 
#' ggplot(df) +
#'   aes(
#'     x, y, label = label, angle = angle, color = color, fill = fill,
#'     hjust = hjust, vjust = vjust
#'   ) +
#'   geom_richtext() +
#'   geom_point(color = "black", size = 2) +
#'   scale_color_identity() +
#'   scale_fill_identity() +
#'   xlim(0, 1) + ylim(0, 1)
#'   
#' # labels without frame or background are also possible
#' ggplot(df) +
#'   aes(
#'     x, y, label = label, angle = angle, color = color,
#'     hjust = hjust, vjust = vjust
#'   ) +
#'   geom_richtext(
#'     fill = NA, label.color = NA, # remove background and outline
#'     label.padding = grid::unit(rep(0, 4), "pt") # remove padding
#'   ) +
#'   geom_point(color = "black", size = 2) +
#'   scale_color_identity() +
#'   xlim(0, 1) + ylim(0, 1)
#' @export
geom_richtext <- function(mapping = NULL, data = NULL,
                      stat = "identity", position = "identity",
                      ...,
                      nudge_x = 0,
                      nudge_y = 0,
                      label.padding = unit(c(0.25, 0.25, 0.25, 0.25), "lines"),
                      label.margin = unit(c(0, 0, 0, 0), "lines"),
                      label.r = unit(0.15, "lines"),
                      na.rm = FALSE,
                      show.legend = NA,
                      inherit.aes = TRUE)
{
  if (!missing(nudge_x) || !missing(nudge_y)) {
    if (!missing(position)) {
      stop("You must specify either `position` or `nudge_x`/`nudge_y` but not both.", call. = FALSE)
    }

    position <- position_nudge(nudge_x, nudge_y)
  }

  layer(
    data = data,
    mapping = mapping,
    stat = stat,
    geom = GeomRichText,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list(
      label.padding = label.padding,
      label.margin = label.margin,
      label.r = label.r,
      na.rm = na.rm,
      ...
    )
  )
}

#' @rdname geom_richtext
#' @format NULL
#' @usage NULL
#' @export
GeomRichText <- ggproto("GeomRichText", Geom,
  required_aes = c("x", "y", "label"),

  default_aes = aes(
    colour = "black", fill = "white", size = 3.88, angle = 0, hjust = 0.5,
    vjust = 0.5, alpha = NA, family = "", fontface = 1, lineheight = 1.2,
    text.colour = NULL, label.colour = NULL, label.size = 0.25
  ),

  draw_panel = function(data, panel_params, coord, 
                        label.padding = unit(c(0.25, 0.25, 0.25, 0.25), "lines"),
                        label.margin = unit(c(0, 0, 0, 0), "lines"),
                        label.r = unit(0.15, "lines"),
                        na.rm = FALSE) {
    data <- coord$transform(data, panel_params)
    if (is.character(data$vjust)) {
      data$vjust <- compute_just(data$vjust, data$y)
    }
    if (is.character(data$hjust)) {
      data$hjust <- compute_just(data$hjust, data$x)
    }

    richtext_grob(
      data$label,
      data$x, data$y, default.units = "native",
      hjust = data$hjust, vjust = data$vjust,
      rot = data$angle,
      padding = label.padding,
      margin = label.margin,
      gp = gpar(
        col = scales::alpha(data$text.colour %||% data$colour, data$alpha),
        fontsize = data$size * .pt,
        fontfamily = data$family,
        fontface = data$fontface,
        lineheight = data$lineheight
      ),
      box_gp = gpar(
        col = scales::alpha(data$label.colour %||% data$colour, data$alpha),
        fill = scales::alpha(data$fill, data$alpha),
        lwd = data$label.size * .pt
      ),
      r = label.r
    )
  },

  draw_key = draw_key_text
)

#' @rdname geom_richtext
#' @format NULL
#' @usage NULL
#' @export
GeomRichtext <- GeomRichText # for automated geom discovery

compute_just <- function(just, x) {
  inward <- just == "inward"
  just[inward] <- c("left", "middle", "right")[just_dir(x[inward])]
  outward <- just == "outward"
  just[outward] <- c("right", "middle", "left")[just_dir(x[outward])]

  unname(c(left = 0, center = 0.5, right = 1,
    bottom = 0, middle = 0.5, top = 1)[just])
}

just_dir <- function(x, tol = 0.001) {
  out <- rep(2L, length(x))
  out[x < 0.5 - tol] <- 1L
  out[x > 0.5 + tol] <- 3L
  out
}

context("geom richtext")

test_that("visual tests", {
  df <- data.frame(
    x = 0.5,
    y = 0.5,
    label = "The **quick** *brown* fox.",
    angle = c(0, 45, 90, 135),
    label.colour = c("#858E4E", "#24988F", "#8C82B8", "#B97681"),
    fill = c("#E6EBD1", "#CDEFE9", "#E8E6FC", "#FEE1E5"),
    label.size = c(0.1, 0.25, 0.5, 1)
  )
  
  p <- ggplot() +
    geom_richtext(
      data = df,
      aes(
        x, y, label = label, label.colour = label.colour, fill = fill,
        label.size = label.size, angle = angle
      ),
      label.margin = grid::unit(rep(0.25, 4), "in"),
      hjust = 0, vjust = 0.5
    ) +
    xlim(0, 1) + ylim(0, 1) +
    scale_discrete_identity(aesthetics = c("orientation", "label.colour", "fill"))
  
  expect_doppelganger("Rotated labels w/ colors", p)
})

#' Theme element that enables markdown text.
#' 
#' Theme element that enables markdown text.
#' 
#' @param family Font family
#' @param face Font face
#' @param size Font size
#' @param colour,color Text color
#' @param fill Fill color of the enclosing box
#' @param box.colour,box.color Line color of the enclosing box (if different from the text color)
#' @param linetype Line type of the enclosing box (like `lty` in base R)
#' @param linewidth Line width of the enclosing box (measured in mm, just like `size` in
#'   [ggplot2::element_line()]).
#' @param hjust Horizontal justification
#' @param vjust Vertical justification
#' @param halign Horizontal justification
#' @param valign Vertical justification
#' @param lineheight Line height
#' @param padding,margin Padding and margins around the text box.
#'   See [gridtext::richtext_grob()] for details.
#' @param r Unit value specifying the corner radius of the box 
#' @param angle Angle (in degrees)
#' @param align_widths,align_heights Should multiple elements be aligned by their
#'   widths or height? See [gridtext::richtext_grob()] for details.
#' @param rotate_margins Should margins get rotated in frame with rotated text?
#'   If `TRUE`, the margins are applied relative to the text direction. If `FALSE`,
#'   the margins are applied relative to the plot direction, i.e., the top margin,
#'   for example, is always placed above the text label, regardless of the direction
#'   in which the text runs. The default is `FALSE`, which mimics the behavior of 
#'   `element_text()`.
#' @param debug Draw a debugging box around each label
#' @param inherit.blank See [ggplot2::margin()] for details.
#' @return A ggplot2 theme element that can be used inside a [ggplot2::theme()]
#'   call.
#' @seealso [gridtext::richtext_grob()], [element_textbox()], [geom_richtext()]
#' @export
element_markdown <- function(family = NULL, face = NULL, size = NULL, colour = NULL, fill = NULL,
                             box.colour = NULL, linetype = NULL, linewidth = NULL,
                             hjust = NULL, vjust = NULL, halign = NULL, valign = NULL, angle = NULL,
                             lineheight = NULL, margin = NULL, padding = NULL, r = NULL, 
                             color = NULL, box.color = NULL, align_widths = NULL,
                             align_heights = NULL, rotate_margins = NULL,
                             debug = FALSE, inherit.blank = FALSE) {
  if (!is.null(color))
    colour <- color
  
  if (!is.null(box.color))
    box.colour <- box.color
  
  structure(
    list(
      family = family, face = face, size = size, colour = colour, fill = fill,
      box.colour = box.colour, linetype = linetype, linewidth = linewidth, 
      hjust = hjust, vjust = vjust, halign = halign, valign = valign, angle = angle,
      lineheight = lineheight, margin = margin, padding = padding, r = r, 
      align_widths = align_widths, align_heights = align_heights, 
      rotate_margins = rotate_margins,
      debug = debug, inherit.blank = inherit.blank),
    class = c("element_markdown", "element_text", "element")
  )
}

#' @export
element_grob.element_markdown <- function(element, label = "", x = NULL, y = NULL,
                                          family = NULL, face = NULL, colour = NULL, size = NULL,
                                          hjust = NULL, vjust = NULL, angle = NULL, lineheight = NULL,
                                          margin = NULL, margin_x = FALSE, margin_y = FALSE, ...) {
  if (is.null(label))
    return(ggplot2::zeroGrob())

  hj <- hjust %||% element$hjust
  vj <- vjust %||% element$vjust
  halign <- element$halign %||% hj
  valign <- element$valign %||% vj
  padding <- element$padding %||% ggplot2::margin(0, 0, 0, 0)
  margin <- margin %||% element$margin %||% ggplot2::margin(0, 0, 0, 0)
  angle <- angle %||% element$angle %||% 0
  r <- element$r %||% unit(0, "pt")
  align_widths <- isTRUE(element$align_widths)
  align_heights <- isTRUE(element$align_heights)
  
  # We rotate the justifiation values to obtain the correct x and y reference point,
  # since box_hjust and box_vjust are applied relative to the rotated text frame in richtext_grob
  just <- rotate_just(angle, hj, vj)
  
  n <- max(length(x), length(y), 1)
  x <- x %||% unit(rep(just$hjust, n), "npc")
  y <- y %||% unit(rep(just$vjust, n), "npc")

  # The gp settings can override element_gp
  gp <- gpar(
    fontsize = size %||% element$size,
    col = colour %||% element$colour,
    fontfamily = family %||% element$family,
    fontface = face %||% element$face,
    lineheight = lineheight %||% element$lineheight
  )
  
  box_gp <- gpar(
    col = element$box.colour %||% gp$col,
    fill = element$fill %||% NA,
    lty = element$linetype %||% 0,
    lwd = (element$linewidth %||% 0.5)*ggplot2::.pt
  )
  
  mrg <- fixup_margins(element$rotate_margins, margin, angle)
  
  if (isTRUE(mrg$native_margins)) {
    richtext_grob(
      label, x = x, y = y, hjust = hj, vjust = vj, halign = halign, valign = valign,
      rot = angle, padding = padding, margin = mrg$margin, r = r, 
      align_widths = align_widths, align_heights = align_heights,
      gp = gp, box_gp = box_gp,
      debug = element$debug
    )
  } else {
    grob <- richtext_grob(
      label, x = x, y = y, hjust = hj, vjust = vj, halign = halign, valign = valign,
      rot = angle, padding = padding, margin = unit(c(0, 0, 0, 0), "pt"), r = r,
      align_widths = align_widths, align_heights = align_heights,
      gp = gp, box_gp = box_gp,
      debug = element$debug
    )
    add_margins(grob, margin, margin_x, margin_y, debug = element$debug)
  }
}


# Modeled after ggplot2 function:
# https://github.com/tidyverse/ggplot2/blob/49d438cf9981f9f0624a27131775ecd3d5bb3455/R/margins.R#L294-L334
# modified here to work with vectorized input
rotate_just <- function(angle, hjust, vjust) {
  angle <- (angle %||% 0) %% 360
  
  hnew <- ifelse(
    0 <= angle & angle < 90,
    hjust,
    ifelse(
      90 <= angle & angle < 180,
      1 - vjust,
      ifelse(
        180 <= angle & angle < 270,
        1 - hjust,
        vjust
      )
    )
  )
  
  vnew <- ifelse(
    0 <= angle & angle < 90,
    vjust,
    ifelse(
      90 <= angle & angle < 180,
      hjust,
      ifelse(
        180 <= angle & angle < 270,
        1 - vjust,
        1 - hjust
      )
    )
  )
  
  list(hjust = hnew, vjust = vnew)
}


fixup_margins <- function(rotate_margins, margin, angle) {
  if (isTRUE(rotate_margins)) {
    return(list(native_margins = TRUE, margin = margin))
  }
  
  angle <- round(angle) %% 360
  
  # if we're given more than one angle or angles corresponding to anything
  # other than horizontal or vertical positions, we cannot use native margins
  if (length(unique(angle)) > 1 || any(!angle %in% c(0, 90, 180, 270))) {
    return(list(native_margins = FALSE, margin = NULL))
  } else {
    angle <- angle[1]
  }
  
  if (angle == 0) {
    return(list(native_margins = TRUE, margin = margin))
  }
  
  if (angle == 90) {
    return(list(native_margins = TRUE, margin = margin[c(4, 1, 2, 3)]))
  }
  
  if (angle == 270) {
    return(list(native_margins = TRUE, margin = margin[c(2, 3, 4, 1)]))
  }
  
  # the only case remaining is angle == 180
  return(list(native_margins = TRUE, margin = margin[c(3, 4, 1, 2)]))
}

context("element markdown")

test_that("ggplot2 margin simulation", {
  # ggplot2 margins are simulated with reshuffled
  # richtext_grob margins for horizontal and vertical
  # text, but not for other angles
  el <- element_markdown(
    size = 12, family = "", face = "plain", colour = "black",
    hjust = 0.5, vjust = 0.5, lineheight = 0,
    margin = margin(10, 10, 10, 10)
  )
  g <- element_grob(
    el, "test", angle = 0, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  g <- element_grob(
    el, "test", angle = 90, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  g <- element_grob(
    el, "test", angle = 180, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  g <- element_grob(
    el, "test", angle = 270, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  g <- element_grob(
    el, "test", angle = 45, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "titleGrob")
  
  # when ggplot2 margins are turned off, result is
  # always a richtext_grob
  el <- element_markdown(
    size = 12, family = "", face = "plain", colour = "black",
    hjust = 0.5, vjust = 0.5, lineheight = 0,
    margin = margin(10, 10, 10, 10),
    rotate_margins = TRUE
  )
  g <- element_grob(
    el, "test", angle = 0, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  g <- element_grob(
    el, "test", angle = 90, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  g <- element_grob(
    el, "test", angle = 180, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  g <- element_grob(
    el, "test", angle = 270, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  
  g <- element_grob(
    el, "test", angle = 45, x = 0.5, y = 0.5,
    margin_x = TRUE, margin_y = TRUE
  )
  expect_s3_class(g, "richtext_grob")
  
})

test_that("visual tests", {
  
  df <- data.frame(x = seq(50, 450, by = 50), y = 9:1)
  
  p <- ggplot(df, aes(x, y)) +
    geom_point() +
    xlab("really really long x label") +
    ylab("long y label")
  
  debug <- FALSE
  
  p1 <- p +
    theme(
      axis.text.x = element_markdown(
        margin = margin(10, 0, 20, 0),
        debug = debug
      ),
      axis.title.y = element_markdown(
        margin = margin(0, 10, 0, 20),
        debug = debug
      )
    )
  
  p2 <- p +
    theme(
      axis.text.x = element_text(
        margin = margin(10, 0, 20, 0),
        debug = debug
      ),
      axis.title.y = element_text(
        margin = margin(0, 10, 0, 20),
        debug = debug
      )
    )
  
  p3 <- p +
    theme(
      axis.text.x = element_markdown(
        hjust = 1,
        margin = margin(10, 0, 20, 0),
        angle = 45,
        debug = debug
      ),
      axis.title.y = element_markdown(
        vjust = 0.5,
        margin = margin(0, 10, 0, 20),
        angle = 180,
        debug = debug
      )
    )
  
  p4 <- p +
    theme(
      axis.text.x = element_text(
        hjust = 1,
        margin = margin(10, 0, 20, 0),
        angle = 45,
        debug = debug
      ),
      axis.title.y = element_text(
        vjust = 0.5,
        margin = margin(0, 10, 0, 20),
        angle = 180,
        debug = debug
      )
    )
  
  expect_doppelganger(
    "Margins match w/ ggtext and ggplot2",
    cowplot::plot_grid(p1, p2, p3, p4)
  )
})

#' Theme element that enables markdown text in a box.
#' 
#' The theme elements `element_textbox()` and `element_textbox_simple()` enable Markdown text in a box, with
#' word wrap. Both functions implement exactly the same functionality; they only differ in the default values
#' for the various element values. `element_textbox()` sets all values that are not specified to `NULL`, as is
#' the usual practice in ggplot2 themes. These missing values are usually completed by inheritance from 
#' parent theme elements. By contrast, `element_textbox_simple()` provides meaningful default values for many of
#' the values that are not usually defined in ggplot2 themes. This makes it simpler to use a textbox element
#' in the context of an existing theme.
#' 
#' @param family Font family
#' @param face Font face
#' @param size Font size (in pt)
#' @param colour,color Text color
#' @param fill Fill color of the enclosing box
#' @param box.colour,box.color Line color of the enclosing box (if different from the text color)
#' @param linetype Line type of the enclosing box (like `lty` in base R)
#' @param linewidth Line width of the enclosing box (measured in mm, just like `size` in
#'   [ggplot2::element_line()]).
#' @param hjust Horizontal justification
#' @param vjust Vertical justification
#' @param halign Horizontal justification
#' @param valign Vertical justification
#' @param lineheight Line height, in multiples of the font size
#' @param width,height Unit objects specifying the width and height
#'   of the textbox, as in [gridtext::textbox_grob()].
#' @param minwidth,minheight,maxwidth,maxheight Min and max values for width and height.
#'   Set to NULL to impose neither a minimum nor a maximum.
#' @param padding,margin Padding and margins around the text box.
#'   See [gridtext::textbox_grob()] for details.
#' @param r Unit value specifying the corner radius of the box 
#' @param orientation Orientation of the text box. See [gridtext::textbox_grob()] for details.
#' @param debug Not implemented.
#' @param inherit.blank See [ggplot2::margin()] for details.
#' @return A ggplot2 theme element that can be used inside a [ggplot2::theme()]
#'   call.
#' @seealso [gridtext::textbox_grob()], [element_markdown()], [geom_textbox()]
#' @examples
#' library(ggplot2)
#' 
#' ggplot(mtcars, aes(disp, mpg)) + 
#' geom_point() +
#'   labs(
#'     title =
#'     "<b><span style = 'font-size:13pt'>Fuel economy vs. engine displacement</span></b><br>
#'     Lorem ipsum *dolor sit amet,* consectetur adipiscing elit, **sed do eiusmod tempor
#'     incididunt** ut labore et dolore magna aliqua. <span style = 'color:red;'>Ut enim ad
#'     minim veniam,</span> quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea
#'     commodo consequat.",
#'     x = "displacement (in<sup>3</sup>)",
#'     y = "Miles per gallon (mpg)<br><span style = 'font-size:8pt'>A measure of
#'     the car's fuel efficiency.</span>"
#'   ) +
#'   theme(
#'     plot.title.position = "plot",
#'     plot.title = element_textbox_simple(
#'       size = 10,
#'       padding = margin(5.5, 5.5, 5.5, 5.5),
#'       margin = margin(0, 0, 5.5, 0),
#'       fill = "cornsilk"
#'     ),
#'     axis.title.x = element_textbox_simple(
#'       width = NULL,
#'       padding = margin(4, 4, 4, 4),
#'       margin = margin(4, 0, 0, 0),
#'       linetype = 1,
#'       r = grid::unit(8, "pt"),
#'       fill = "azure1"
#'     ),
#'     axis.title.y = element_textbox_simple(
#'       hjust = 0,
#'       orientation = "left-rotated",
#'       minwidth = unit(1, "in"),
#'       maxwidth = unit(2, "in"),
#'       padding = margin(4, 4, 2, 4),
#'       margin = margin(0, 0, 2, 0),
#'       fill = "lightsteelblue1"
#'     )
#'   )
#' @export
element_textbox <- function(family = NULL, face = NULL, size = NULL, colour = NULL, fill = NULL,
                            box.colour = NULL, linetype = NULL, linewidth = NULL,
                            hjust = NULL, vjust = NULL, halign = NULL, valign = NULL, lineheight = NULL,
                            margin = NULL, padding = NULL, width = NULL, height = NULL, minwidth = NULL,
                            maxwidth = NULL, minheight = NULL, maxheight = NULL, r = NULL,
                            orientation = NULL, color = NULL, box.color = NULL,
                            debug = FALSE, inherit.blank = FALSE) {
  if (!is.null(color))
    colour <- color
  
  if (!is.null(box.color))
    box.colour <- box.color
  
  structure(
    list(
      family = family, face = face, size = size, colour = colour, fill = fill, box.colour = box.colour,
      linetype = linetype, linewidth = linewidth, 
      hjust = hjust, vjust = vjust, halign = halign, valign = valign, lineheight = lineheight, 
      margin = margin, padding = padding, width = width, height = height, minwidth = minwidth,
      maxwidth = maxwidth, minheight = minheight, maxheight = maxheight,
      r = r, orientation = orientation,
      debug = debug, inherit.blank = inherit.blank),
    class = c("element_textbox", "element_text", "element")
  )
}

#' @rdname element_textbox
#' @export
element_textbox_simple <- function(family = NULL, face = NULL, size = NULL, colour = NULL, fill = NA,
                            box.colour = NULL, linetype = 0, linewidth = 0.5,
                            hjust = 0.5, vjust = 0.5, halign = 0, valign = 1, lineheight = 1.2,
                            margin = ggplot2::margin(0, 0, 0, 0), padding = ggplot2::margin(0, 0, 0, 0),
                            width = grid::unit(1, "npc"), height = NULL, minwidth = NULL,
                            maxwidth = NULL, minheight = NULL, maxheight = NULL, r = grid::unit(0, "pt"),
                            orientation = "upright", color = NULL, box.color = NULL,
                            debug = FALSE, inherit.blank = FALSE) {
  element_textbox(
    family = family, face = face, size = size, colour = colour, fill = fill,
    box.colour = box.colour, linetype = linetype, linewidth = linewidth, hjust = hjust, vjust = vjust,
    halign = halign, valign = valign, lineheight = lineheight, margin = margin, padding = padding,
    width = width, height = height, minwidth = minwidth, maxwidth = maxwidth, minheight = minheight,
    maxheight = maxheight, r = r, orientation = orientation, color = color, box.color = box.color
  )
}

#' @export
element_grob.element_textbox <- function(element, label = "", x = NULL, y = NULL,
                                         family = NULL, face = NULL, colour = NULL, size = NULL,
                                         hjust = NULL, vjust = NULL, lineheight = NULL,
                                         margin = NULL, ...) {
  if (is.null(label))
    return(ggplot2::zeroGrob())

  hj <- hjust %||% element$hjust
  vj <- vjust %||% element$vjust
  halign <- element$halign %||% 0
  valign <- element$valign %||% 1
  padding <- element$padding %||% ggplot2::margin(0, 0, 0, 0)
  margin <- margin %||% element$margin %||% ggplot2::margin(0, 0, 0, 0)
  orientation <- element$orientation %||% "upright"
  r <- element$r %||% unit(0, "pt")
  
  # The gp settings can override element_gp
  gp <- gpar(
    fontsize = size %||% element$size,
    col = colour %||% element$colour,
    fontfamily = family %||% element$family,
    fontface = face %||% element$face,
    lineheight = lineheight %||% element$lineheight
  )

  box_gp <- gpar(
    col = element$box.colour %||% gp$col,
    fill = element$fill %||% NA,
    lty = element$linetype %||% 0,
    lwd = (element$linewidth %||% 0.5)*ggplot2::.pt
  )
  
  textbox_grob(
    label, x = x, y = y, hjust = hj, vjust = vj, halign = halign, valign = valign,
    width = element$width, height = element$height,
    minwidth = element$minwidth, minheight = element$minheight,
    maxwidth = element$maxwidth, maxheight = element$maxheight,
    margin = margin, padding = padding, r = r,
    orientation = orientation, 
    gp = gp, box_gp = box_gp
  )
}

context("element textbox")

test_that("visual tests", {
  text <- "*Lorem ipsum dolor sit amet,* consectetur adipiscing
elit. **Quisque tincidunt** eget arcu in pulvinar. Morbi varius leo
vel consectetur luctus. **Morbi facilisis justo non fringilla.**
Vivamus sagittis sem felis, vel lobortis risus mattis eget. Nam
quis imperdiet felis, in convallis elit."
  
  base <- ggplot(data.frame(x = 1:3), aes(x, x)) + geom_point()
  
  p <- base +
    labs(subtitle = text) +
    theme(
      plot.title.position = "plot",
      plot.subtitle = element_textbox_simple(
        size = 11, lineheight = 1.2, padding = margin(0, 0, 5, 0)
      )
    )
  
  expect_doppelganger("Simple textbox as plot title", p)
  
  p <- base +
    labs(subtitle = text) +
    theme(
      plot.title.position = "plot",
      plot.subtitle = element_textbox_simple(
        size = 11, lineheight = 1.2,
        linetype = 1, # turn on border
        box.color = "#748696", # border color
        fill = "#F0F7FF", # background fill color
        r = grid::unit(3, "pt"), # radius for rounded corners
        padding = margin(5, 5, 5, 5), # padding around text inside the box
        margin = margin(0, 0, 10, 0) # margin outside the box
      )
    )
  
  expect_doppelganger("Plot title with styling", p)
  
  p <- base +
    labs(subtitle = text) +
    theme(
      plot.title.position = "plot",
      plot.subtitle = element_textbox_simple(
        size = 11, lineheight = 1.2,
        width = grid::unit(4, "in"), # fixed width
        hjust = 1, # alignment of box relative to plot
        linetype = 1, # turn on border
        box.color = "#748696", # border color
        fill = "#F0F7FF", # background fill color
        r = grid::unit(3, "pt"), # radius for rounded corners
        padding = margin(5, 5, 5, 5), # padding around text inside the box
        margin = margin(0, 0, 10, 0) # margin outside the box
      )
    )
  
  expect_doppelganger("Plot title with fixed width", p)
  
  p <- base +
    labs(y = text) +
    theme(
      axis.title.y = element_textbox_simple(
        size = 11, lineheight = 1.2,
        orientation = "left-rotated",
        width = grid::unit(2.5, "in"),
        hjust = 0, fill = "#F0F7FF",  
        padding = margin(5, 5, 5, 5),
        margin = margin(0, 0, 10, 0)
      )
    )
  
  expect_doppelganger("Rotated box as y axis title", p)
})

#' Draw boxes containing text
#'
#' Draw boxes of defined width and height containing word-wrapped text. Multiple
#' boxes can be drawn at once. Most styling parameters can be used as aesthetics
#' and can be applied separately to each text box drawn. The exception is styling
#' parameters that are specified as grid units (e.g., `box.padding` or `box.r`),
#' which can only be specified for all text boxes at once. See examples for details.
#' 
#' @section Aesthetics:
#' 
#' `geom_textbox()` understands the following aesthetics (required 
#' aesthetics are in bold; select aesthetics are annotated):
#' 
#' * **`x`**
#' * **`y`**
#' * **`label`**
#' * `alpha`
#' * `box.colour` Color of box outline. Overrides `colour`.
#' * `box.size` Width of box outline.
#' * `colour` Default color of box text and box outline.
#' * `family`
#' * `fontface`
#' * `fill` Default fill color of box background.
#' * `group`
#' * `halign` Horizontal alignment of text inside box.
#' * `hjust` Horizontal alignment of box.
#' * `lineheight`
#' * `orientation` One of `"upright"`, `"left-rotated"`,
#'     `"right-rotated"`, `"inverted"`.
#' * `size` Default font size of box text.
#' * `text.colour` Color of box text. Overrides `colour`.
#' * `valign` Vertical alignment of text inside box.
#' * `vjust` Vertical alignment of box.
#'
#' @inheritParams ggplot2::geom_text
#' @param nudge_x,nudge_y Horizontal and vertical adjustment to nudge text boxes by.
#'   Useful for offsetting text from points, particularly on discrete scales.
#'   Cannot be jointly specified with `position`.
#' @param width,height Unit values specifying the width and height of
#'   the text box (including margins!). If `height = NULL` (the default),
#'   the height is chosen automatically to accommodate all the text.
#' @param minwidth,maxwidth,minheight,maxheight Unit values specifying
#'   the minimum and maximum values for `width` and `height`, respectively.
#'   If set to `NULL`, are not enforced.
#' @param box.padding Unit vector of length four specifying the padding
#'   inside the text box.
#' @param box.margin Unit vector of length four specifying the margin
#'   outside the text box.
#' @param box.r Unit vector of length one specifying the radius of the
#'   box.
#' @return A ggplot2 layer that can be added to a plot created with
#'   [ggplot2::ggplot()].
#' @seealso [geom_richtext()], [element_textbox()]
#' @examples
#' library(ggplot2)
#' 
#' df <- data.frame(
#'   label = rep("Lorem ipsum dolor **sit amet,** consectetur adipiscing elit,
#'     sed do *eiusmod tempor incididunt* ut labore et dolore magna
#'     aliqua.", 2),
#'   x = c(0, .6),
#'   y = c(1, .6),
#'   hjust = c(0, 0),
#'   vjust = c(1, 0),
#'   orientation = c("upright", "right-rotated"),
#'   color = c("black", "blue"),
#'   fill = c("cornsilk", "white")
#' )
#' 
#' ggplot(df) +
#'   aes(
#'     x, y, label = label, color = color, fill = fill,
#'     hjust = hjust, vjust = vjust,
#'     orientation = orientation
#'   ) +
#'   geom_textbox(width = unit(0.4, "npc")) +
#'   geom_point(color = "black", size = 2) +
#'   scale_discrete_identity(aesthetics = c("color", "fill", "orientation")) +
#'   xlim(0, 1) + ylim(0, 1)
#' @export
geom_textbox <- function(mapping = NULL, data = NULL,
                         stat = "identity", position = "identity",
                         ...,
                         nudge_x = 0,
                         nudge_y = 0,
                         box.padding = unit(c(5.5, 5.5, 5.5, 5.5), "pt"),
                         box.margin = unit(c(0, 0, 0, 0), "pt"),
                         box.r = unit(5.5, "pt"),
                         width = unit(2, "inch"), minwidth = NULL, maxwidth = NULL,
                         height = NULL, minheight = NULL, maxheight = NULL,
                         na.rm = FALSE,
                         show.legend = NA,
                         inherit.aes = TRUE)
{
  if (!missing(nudge_x) || !missing(nudge_y)) {
    if (!missing(position)) {
      stop("You must specify either `position` or `nudge_x`/`nudge_y` but not both.", call. = FALSE)
    }

    position <- position_nudge(nudge_x, nudge_y)
  }

  layer(
    data = data,
    mapping = mapping,
    stat = stat,
    geom = GeomTextBox,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list(
      box.padding = box.padding,
      box.margin = box.margin,
      box.r = box.r,
      width = width, minwidth = minwidth, maxwidth = maxwidth,
      height = height, minheight = minheight, maxheight = maxheight,
      na.rm = na.rm,
      ...
    )
  )
}

#' @rdname geom_richtext
#' @format NULL
#' @usage NULL
#' @export
GeomTextBox <- ggproto("GeomTextBox", Geom,
  required_aes = c("x", "y", "label"),

  default_aes = aes(
    colour = "black", fill = "white", size = 3.88, hjust = 0.5,
    vjust = 0.5, halign = 0, valign = 1, alpha = NA, family = "",
    fontface = 1, lineheight = 1.2, text.colour = NULL, box.colour = NULL,
    box.size = 0.25, orientation = "upright"
  ),

  draw_panel = function(data, panel_params, coord, 
                        box.padding = unit(c(5.5, 5.5, 5.5, 5.5), "pt"),
                        box.margin = unit(c(0, 0, 0, 0), "pt"),
                        box.r = unit(5.5, "pt"),
                        width = unit(2, "inch"), minwidth = NULL, maxwidth = NULL,
                        height = NULL, minheight = NULL, maxheight = NULL,
                        na.rm = FALSE) {
    data <- coord$transform(data, panel_params)
    
    # split data frame into list of rows
    rows <- split(data, seq(nrow(data)))
    names(rows) <- NULL
    grobs <- mapply(
      make_textbox_grob,
      rows,
      list(box.padding),
      list(box.margin),
      list(box.r),
      list(width),
      list(minwidth),
      list(maxwidth),
      list(height),
      list(minheight),
      list(maxheight),
      SIMPLIFY = FALSE
    )
    do.call(grobTree, grobs)
  },

  draw_key = draw_key_text
)

make_textbox_grob <- function(data, 
                              box.padding = unit(c(5.5, 5.5, 5.5, 5.5), "pt"),
                              box.margin = unit(c(0, 0, 0, 0), "pt"),
                              box.r = unit(5.5, "pt"),
                              width = unit(2, "inch"), minwidth = NULL, maxwidth = NULL,
                              height = NULL, minheight = NULL, maxheight = NULL) {
  textbox_grob(
    data$label,
    data$x, data$y, default.units = "native",
    hjust = data$hjust, vjust = data$vjust,
    halign = data$halign, valign = data$valign,
    orientation = data$orientation,
    padding = box.padding,
    margin = box.margin,
    width = width, minwidth = minwidth, maxwidth = maxwidth,
    height = height, minheight = minheight, maxheight = maxheight,
    gp = gpar(
      col = scales::alpha(data$text.colour %||% data$colour, data$alpha),
      fontsize = data$size * .pt,
      fontfamily = data$family,
      fontface = data$fontface,
      lineheight = data$lineheight
    ),
    box_gp = gpar(
      col = scales::alpha(data$box.colour %||% data$colour, data$alpha),
      fill = scales::alpha(data$fill, data$alpha),
      lwd = data$box.size * .pt
    ),
    r = box.r
  )
}

context("geom textbox")

test_that("visual tests", {
  df <- data.frame(
    x = 0.5,
    y = 0.5,
    label = "The quick brown fox **jumps over** the *lazy* dog.",
    orientation = c("upright", "left-rotated", "inverted", "right-rotated"),
    box.colour = c("#858E4E", "#24988F", "#8C82B8", "#B97681"),
    fill = c("#E6EBD1", "#CDEFE9", "#E8E6FC", "#FEE1E5"),
    halign = c(0, 0.5, 1, 0.5),
    valign = c(1, 0.5, 0.5, 0),
    box.size = c(0.1, 0.25, 0.5, 1)
  )
  
  p <- ggplot() +
    geom_textbox(
      data = df,
      aes(
        x, y, label = label, box.colour = box.colour, fill = fill,
        halign = halign, valign = valign, box.size = box.size,
        orientation = orientation
      ),
      width = grid::unit(1.5, "in"),
      height = grid::unit(1.5, "in"),
      box.margin = grid::unit(rep(0.25, 4), "in"),
      hjust = 0, vjust = 1
    ) +
    xlim(0, 1) + ylim(0, 1) +
    scale_discrete_identity(aesthetics = c("orientation", "box.colour", "fill"))
  
  expect_doppelganger("Rotated boxes w/ colors and alignments", p)
})

# modified from ggplot2 function add_margins()
add_margins <- function(grob, margin = NULL, margin_x = FALSE, margin_y = FALSE, debug = FALSE) {
  if (is.null(margin)) {
    margin <- margin(0, 0, 0, 0)
  }
  
  width <- grobWidth(grob)
  height <- grobHeight(grob)
  
  if (margin_x && margin_y) {
    widths <- unit.c(margin[4], width, margin[2])
    heights <- unit.c(margin[1], height, margin[3])
    
    vp <- viewport(
      layout = grid.layout(3, 3, heights = heights, widths = widths)
    )
    child_vp <- viewport(layout.pos.row = 2, layout.pos.col = 2)
  } else if (margin_x) {
    widths <- unit.c(margin[4], width, margin[2])
    heights <- unit(1, "null")
    
    vp <- viewport(layout = grid.layout(1, 3, widths = widths))
    child_vp <- viewport(layout.pos.col = 2)
  } else if (margin_y) {
    widths <- unit(1, "null")
    heights <- unit.c(margin[1], height, margin[3])
    
    vp <- viewport(layout = grid.layout(3, 1, heights = heights))
    child_vp <- viewport(layout.pos.row = 2)
  } else {
    widths <- width
    heights <- height
    
    if (isTRUE(debug)) {
      bg <- rectGrob(gp = gpar(fill = "cornsilk"))
      g <- gTree(
        children = gList(bg, grob),
        widths = widths,
        heights = heights,
        cl = "titleGrob"
      )
    } else {
      g <- gTree(
        children = gList(grob),
        widths = widths,
        heights = heights,
        cl = "titleGrob"
      )
    }
    return(g)
  }
  
  if (isTRUE(debug)) {
    bg <- rectGrob(gp = gpar(fill = "cornsilk", col = NA))
    gTree(
      children = gList(bg, grob),
      vp = vpTree(vp, vpList(child_vp)),
      widths = widths,
      heights = heights,
      cl = "titleGrob"
    )
  }
  else {
    gTree(
      children = gList(grob),
      vp = vpTree(vp, vpList(child_vp)),
      widths = widths,
      heights = heights,
      cl = "titleGrob"
    )
  }
}

---
title: "Markdown theme elements"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Markdown theme elements}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 4,
  fig.height = 3.75
)
```


The ggtext package defines two new theme elements, `element_markdown()` and `element_textbox()`/`element_textbox_simple()`, which can be used in place of `element_text()` in ggplot2 themes.

### Simple text labels

Simple text labels are created with `element_markdown()`. To demonstrate typical usage, let's start with a basic plot of a parabola.

```{r message = FALSE}
library(ggplot2)
library(ggtext)

base <- ggplot(data.frame(x = c(-5, 5)), aes(x)) +
  stat_function(fun = ~ .x*.x)

base
```

This plot would benefit from nicer axis labels. In particular, assume we want the x axis label to read "independent variable *x*" and the y axis label to read "dependent variable *y* = *x*<sup>2</sup>". In Markdown, we could write the axis labels as `independent variable *x*` and `dependent variable *y* = *x*<sup>2</sup>`. However, if we do so, we need to tell ggplot2 to interpret the axis labels as Markdown and not as plain text. We do this by setting `axis.title.x` and `axis.title.y` to `element_markdown()`. (Note that both are set to `element_text()` in the default theme.)

```{r}
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )
```

The new element `element_markdown()` behaves just like `element_text()`. For example, we can modify the color or the font size.

```{r}
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title.x = element_markdown(color = "blue"),
    axis.title.y = element_markdown(size = rel(0.8))
  )
```

Inheritance of theme settings also works. For example, we can set both color and font size for `axis.title`, and then both `axis.title.x` and `axis.title.y` inherit the setting.

```{r}
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )
```

Note that we used `element_text()` instead of `element_markdown()` for `axis.title` in the above plot.  We could have used `element_markdown()` as well and the result would have been the same. It doesn't matter that we set `axis.title = element_text()`, because the `axis.title` element isn't actually rendered, only the `axis.title.x` and `axis.title.y` elements are. We're setting `axis.title` only for the purpose of providing shared parameter values to `axis.title.x` and `axis.title.y`.

This is important to keep in mind when trying to create more unusual plots, e.g. with the y axis on the right. The naive code fails:

```{r}
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  scale_y_continuous(position = "right") +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )
```

This happens because the axis title on the right is actually drawn by `axis.title.y.right`. Therefore, setting that element to `element_markdown()` creates the desired result.

```{r}
base +
  labs(
    x = "independent variable *x*",
    y = "dependent variable *y* = *x*<sup>2</sup>"
  ) +
  scale_y_continuous(position = "right") +
  theme(
    axis.title = element_text(color = "blue", size = rel(0.8)),
    axis.title.x = element_markdown(),
    axis.title.y.right = element_markdown()
  )
```

Additional styling can be applied via inline CSS. The CSS properties `color`, `font-size`, and `font-family` are currently supported. Multi-line labels can be created by placing `<br>` tags where line breaks are desired.

```{r fig.width = 5, message = FALSE}
library(dplyr)

mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG)",
    title = "<span style = 'font-size:14pt; font-family:Helvetica;'>Transmission type impacts fuel efficiency</span><br>
MPG is higher for <span style = 'color:#0072B2;'>automatic</span>
than for <span style = 'color:#D55E00;'>manual</span> transmissions"
  ) +
  theme_bw() +
  theme(
    text = element_text(family = "Times"),
    plot.title.position = "plot",
    plot.title = element_markdown(size = 11, lineheight = 1.2)
  )

```

We set the `lineheight` property to 1.2 because the default lineheight is too small for multi-line text labels rendered with `element_markdown()`.

### Text boxes

Text boxes can be created with `element_textbox()` or `element_textbox_simple()`. Text boxes differ from labels created with `element_markdown()` in that they tend to have a specific width and wrap their contents so it fits that width. The height of a textbox is normally calculated automatically so it matches the content height, but explicitly setting the height is also possible. Finally, while markdown labels can be displayed at any angle, textboxes have only four possible orientations, upright, left-rotated, right-rotated, and inverted.

In practice, setting a theme element to `element_textbox()` in a ggplot2 theme will frequently not have the desired result, because textboxes require many additional parameters that are not set by the parent text elements present in standard themes. To work around this issue, you can use `element_textbox_simple()`. It sets reasonable defaults for the additional parameters and thus can be used more readily. You will usually be able to use `element_textbox_simple()` as is, with only a few parameter adjustments required. 

The following example adds both a title and a subtitle to the plot by drawing one single text box.

```{r fig.width = 5, message = FALSE}
base <- mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG)",
    title = "Transmission type impacts fuel efficiency<br>
<span style = 'font-size:10pt;'>Miles per gallon (MPG) is on average higher for cars
with <span style = 'color:#0072B2;'>automatic transmission</span> than for cars with
<span style = 'color:#D55E00;'>manual transmission.</span> However, MPG generally
declines with increasing horse power.</span>"
  ) +
  theme_bw() + theme(plot.title.position = "plot")

base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1, padding = margin(0, 0, 5, 0)
    )
  )
```

Text boxes can have a background color and a border, and they have internal padding and external margins.

```{r fig.width = 5, message = FALSE}
base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1,
      linetype = 1, # turn on border
      box.color = "#748696", # border color
      fill = "#F0F7FF", # background fill color
      r = grid::unit(3, "pt"), # radius for rounded corners
      padding = margin(5, 5, 5, 5), # padding around text inside the box
      margin = margin(0, 0, 10, 0) # margin outside the box
    )
  )
```

We can explicitly restrict the width of the box, and we can align the box relative to the enclosing space (with `hjust` and `vjust`) and the box content relative to the box edges (with `halign` and `valign`, not shown).

```{r fig.width = 5, message = FALSE}
base +
  theme(
    plot.title = element_textbox_simple(
      size = 14, lineheight = 1, 
      width = grid::unit(4, "in"), # fixed width
      hjust = 1, # alignment of box relative to plot
      linetype = 1, # turn on border
      box.color = "#748696", # border color
      fill = "#F0F7FF", # background fill color
      r = grid::unit(3, "pt"), # radius for rounded corners
      padding = margin(5, 5, 5, 5), # padding around text inside the box
      margin = margin(0, 0, 10, 0) # margin outside the box
    )
  )
```

If we want a box to be rotated, we can use the `orientation` parameter.

```{r}
mtcars %>%
  mutate(
    transmission = ifelse(am == 1, "automatic", "manual")
  ) %>%
  ggplot(aes(hp, mpg, color = transmission)) +
  geom_point(size = 2) +
  scale_color_manual(
    values = c(automatic = "#0072B2", manual = "#D55E00"),
    guide = "none"
  ) +
  labs(
    x = "Horse power",
    y = "Miles per gallon (MPG) is on average higher for cars with <span style =
'color:#0072B2;'>automatic transmission</span> than for cars with <span style =
'color:#D55E00;'>manual transmission.</span>",
    title = "Transmission type impacts fuel efficiency"
  ) +
  theme_bw() + 
  theme(
    plot.title.position = "plot",
    axis.title.y = element_textbox_simple(
      orientation = "left-rotated",
      width = grid::unit(2.5, "in"),
      hjust = 0, fill = "#F0F7FF",  
      padding = margin(5, 5, 5, 5),
      margin = margin(0, 0, 10, 0)
    )
  )
```

---
title: "Plotting with markdown text"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Plotting with markdown text}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 4,
  fig.height = 3.75
)
```


The ggtext package defines two new geoms, `geom_richtext()` and `geom_textbox()`, which can be used to plot with markdown text. They draw simple text labels (without word wrap) and textboxes (with word wrap), respectively.

### Simple text labels

Markdown-formatted text labels can be placed into a plot with `geom_richtext()`. This geom is mostly a drop-in replacement for `geom_label()` (or `geom_text()`), with added capabilities.

As a first example, we will annotate a plot of linear regressions with their *r*<sup>2</sup> values. We will use the `iris` dataset for this demonstration. In our first iteration, we will not yet use any ggtext features, and instead plot the text with `geom_text()`.


```{r fig.width = 6, fig.height = 3, message = FALSE}
library(ggplot2)
library(dplyr)
library(glue)

iris_cor <- iris %>% 
  group_by(Species) %>%
  summarize(r_square = cor(Sepal.Length, Sepal.Width)^2) %>%
  mutate(
    # location of each text label in data coordinates
    Sepal.Length = 8, Sepal.Width = 4.5,
    # text label containing r^2 value 
    label = glue("r^2 = {round(r_square, 2)}")
  )

iris_cor

iris_facets <- ggplot(iris, aes(Sepal.Length, Sepal.Width)) +
  geom_point() + 
  geom_smooth(method = "lm", formula = y ~ x) +
  facet_wrap(~Species) +
  theme_bw()

iris_facets + 
  geom_text(
    data = iris_cor,
    aes(label = label),
    hjust = 1, vjust = 1
  )
```

This code works, but the result is not fully satisfying. First, because *r* is a mathematical variable, it should be typeset in italics. Second, it would be nicer to have a superscript 2 instead of ^2. We can achieve both results by creating a markdown label and plotting it with `geom_richtext()`.

```{r fig.width = 6, fig.height = 3, message = FALSE}
library(ggtext)

iris_cor_md <- iris_cor %>% 
  mutate(
    # markdown version of text label
    label = glue("*r*<sup>2</sup> = {round(r_square, 2)}")
  )

iris_cor_md

iris_facets + 
  geom_richtext(
    data = iris_cor_md,
    aes(label = label),
    hjust = 1, vjust = 1
  )
```

By default, `geom_richtext()` puts a box around the text it draws. We can suppress the box by setting the fill and outline colors to transparent (`fill = NA, label.colour = NA`). 

```{r fig.width = 6, fig.height = 3}
iris_facets + 
  geom_richtext(
    data = iris_cor_md,
    aes(label = label),
    hjust = 1, vjust = 1,
    # remove label background and outline
    fill = NA, label.color = NA,
    # remove label padding, since we have removed the label outline
    label.padding = grid::unit(rep(0, 4), "pt") 
  )
```

We can separately choose the colors of label outline, label fill, and label text, and we can assign them via aesthetic mapping as well as by direct specification, as is usual in ggplot2.

```{r fig.width = 6, fig.height = 3}
iris_facets + 
  aes(colour = Species) +
  geom_richtext(
    data = iris_cor_md,
    aes(
      label = label,
      fill = after_scale(alpha(colour, .2))
    ),
    text.colour = "black",
    hjust = 1, vjust = 1
  ) +
  theme(legend.position = "none")
```

Rotated labels are also possible, though in most cases it is not recommended to use them.

```{r fig.width = 6, fig.height = 3}
iris_facets + 
  aes(colour = Species) +
  geom_richtext(
    data = iris_cor_md,
    aes(
      x = 7.5,
      label = label,
      fill = after_scale(alpha(colour, .2))
    ),
    text.colour = "black",
    hjust = 1, vjust = 1,
    angle = 30
  ) +
  theme(legend.position = "none")
```

### Text boxes

Markdown-formatted text boxes (with word wrap) can be placed into a plot with `geom_textbox()`. It is generally necessary to specify a width for the box. Widths are specified in grid units, and both absolute (e.g., `"cm"`, `"pt"`, or `"in"`) and relative (`"npc"`, Normalised Parent Coordinates) units are possible.

```{r fig.width = 6, fig.height = 3}
df <- data.frame(
  x = 0.1,
  y = 0.8,
  label = "*Lorem ipsum dolor sit amet,* consectetur adipiscing
elit. Quisque tincidunt eget arcu in pulvinar. Morbi varius leo
vel consectetur luctus. **Morbi facilisis justo non fringilla.**
Vivamus sagittis sem felis, vel lobortis risus mattis eget. Nam
quis imperdiet felis, in convallis elit."
)

p <- ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label),
    width = grid::unit(0.73, "npc"), # 73% of plot panel width
    hjust = 0, vjust = 1
  ) +
  xlim(0, 1) + ylim(0, 1)

p
```

If we specify a relative width, then changing the size of the plot will change the size of the textbox. The text will reflow to accommodate this change.
```{r fig.width = 4, fig.height = 4}
p
```

The parameters `hjust` and `vjust` align the box relative to the reference point specified by `x` and `y`, but they do not affect the alignment of text inside the box. To specify how text is aligned inside the box, use `halign` and `valign`. For example, `halign = 0.5` generates centered text.

```{r fig.width = 4, fig.height = 4}
ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label),
    width = grid::unit(0.73, "npc"), # 73% of plot panel width
    hjust = 0, vjust = 1,
    halign = 0.5 # centered text
  ) +
  xlim(0, 1) + ylim(0, 1)
```

While text boxes cannot be rotated arbitrarily, they can be placed in four distinct orientations, corresponding to rotations by multiples of 90 degrees. Note that `hjust` and `vjust` are specified relative to this orientation.

```{r}
df <- data.frame(
  x = 0.5,
  y = 0.5,
  label = "The quick brown fox jumps over the lazy dog.",
  orientation = c("upright", "left-rotated", "inverted", "right-rotated")
)

ggplot() +
  geom_textbox(
    data = df,
    aes(x, y, label = label, orientation = orientation),
    width = grid::unit(1.5, "in"),
    height = grid::unit(1.5, "in"),
    box.margin = grid::unit(rep(0.25, 4), "in"),
    hjust = 0, vjust = 1
  ) +
  xlim(0, 1) + ylim(0, 1) +
  scale_discrete_identity(aesthetics = "orientation")

```

The previous example uses the `box.margin` argument to create some space between the reference point given by `x`, `y` and the box itself. This margin is part of the size calculation for the box, so that a width of 1.5 inches with 0.25 inch margins yields an actual box of 1 inch in width.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/element-textbox.R
\name{element_textbox}
\alias{element_textbox}
\alias{element_textbox_simple}
\title{Theme element that enables markdown text in a box.}
\usage{
element_textbox(
  family = NULL,
  face = NULL,
  size = NULL,
  colour = NULL,
  fill = NULL,
  box.colour = NULL,
  linetype = NULL,
  linewidth = NULL,
  hjust = NULL,
  vjust = NULL,
  halign = NULL,
  valign = NULL,
  lineheight = NULL,
  margin = NULL,
  padding = NULL,
  width = NULL,
  height = NULL,
  minwidth = NULL,
  maxwidth = NULL,
  minheight = NULL,
  maxheight = NULL,
  r = NULL,
  orientation = NULL,
  color = NULL,
  box.color = NULL,
  debug = FALSE,
  inherit.blank = FALSE
)

element_textbox_simple(
  family = NULL,
  face = NULL,
  size = NULL,
  colour = NULL,
  fill = NA,
  box.colour = NULL,
  linetype = 0,
  linewidth = 0.5,
  hjust = 0.5,
  vjust = 0.5,
  halign = 0,
  valign = 1,
  lineheight = 1.2,
  margin = ggplot2::margin(0, 0, 0, 0),
  padding = ggplot2::margin(0, 0, 0, 0),
  width = grid::unit(1, "npc"),
  height = NULL,
  minwidth = NULL,
  maxwidth = NULL,
  minheight = NULL,
  maxheight = NULL,
  r = grid::unit(0, "pt"),
  orientation = "upright",
  color = NULL,
  box.color = NULL,
  debug = FALSE,
  inherit.blank = FALSE
)
}
\arguments{
\item{family}{Font family}

\item{face}{Font face}

\item{size}{Font size (in pt)}

\item{colour, color}{Text color}

\item{fill}{Fill color of the enclosing box}

\item{box.colour, box.color}{Line color of the enclosing box (if different from the text color)}

\item{linetype}{Line type of the enclosing box (like \code{lty} in base R)}

\item{linewidth}{Line width of the enclosing box (measured in mm, just like \code{size} in
\code{\link[ggplot2:element]{ggplot2::element_line()}}).}

\item{hjust}{Horizontal justification}

\item{vjust}{Vertical justification}

\item{halign}{Horizontal justification}

\item{valign}{Vertical justification}

\item{lineheight}{Line height, in multiples of the font size}

\item{padding, margin}{Padding and margins around the text box.
See \code{\link[gridtext:textbox_grob]{gridtext::textbox_grob()}} for details.}

\item{width, height}{Unit objects specifying the width and height
of the textbox, as in \code{\link[gridtext:textbox_grob]{gridtext::textbox_grob()}}.}

\item{minwidth, minheight, maxwidth, maxheight}{Min and max values for width and height.
Set to NULL to impose neither a minimum nor a maximum.}

\item{r}{Unit value specifying the corner radius of the box}

\item{orientation}{Orientation of the text box. See \code{\link[gridtext:textbox_grob]{gridtext::textbox_grob()}} for details.}

\item{debug}{Not implemented.}

\item{inherit.blank}{See \code{\link[ggplot2:element]{ggplot2::margin()}} for details.}
}
\value{
A ggplot2 theme element that can be used inside a \code{\link[ggplot2:theme]{ggplot2::theme()}}
call.
}
\description{
The theme elements \code{element_textbox()} and \code{element_textbox_simple()} enable Markdown text in a box, with
word wrap. Both functions implement exactly the same functionality; they only differ in the default values
for the various element values. \code{element_textbox()} sets all values that are not specified to \code{NULL}, as is
the usual practice in ggplot2 themes. These missing values are usually completed by inheritance from
parent theme elements. By contrast, \code{element_textbox_simple()} provides meaningful default values for many of
the values that are not usually defined in ggplot2 themes. This makes it simpler to use a textbox element
in the context of an existing theme.
}
\examples{
library(ggplot2)

ggplot(mtcars, aes(disp, mpg)) + 
geom_point() +
  labs(
    title =
    "<b><span style = 'font-size:13pt'>Fuel economy vs. engine displacement</span></b><br>
    Lorem ipsum *dolor sit amet,* consectetur adipiscing elit, **sed do eiusmod tempor
    incididunt** ut labore et dolore magna aliqua. <span style = 'color:red;'>Ut enim ad
    minim veniam,</span> quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea
    commodo consequat.",
    x = "displacement (in<sup>3</sup>)",
    y = "Miles per gallon (mpg)<br><span style = 'font-size:8pt'>A measure of
    the car's fuel efficiency.</span>"
  ) +
  theme(
    plot.title.position = "plot",
    plot.title = element_textbox_simple(
      size = 10,
      padding = margin(5.5, 5.5, 5.5, 5.5),
      margin = margin(0, 0, 5.5, 0),
      fill = "cornsilk"
    ),
    axis.title.x = element_textbox_simple(
      width = NULL,
      padding = margin(4, 4, 4, 4),
      margin = margin(4, 0, 0, 0),
      linetype = 1,
      r = grid::unit(8, "pt"),
      fill = "azure1"
    ),
    axis.title.y = element_textbox_simple(
      hjust = 0,
      orientation = "left-rotated",
      minwidth = unit(1, "in"),
      maxwidth = unit(2, "in"),
      padding = margin(4, 4, 2, 4),
      margin = margin(0, 0, 2, 0),
      fill = "lightsteelblue1"
    )
  )
}
\seealso{
\code{\link[gridtext:textbox_grob]{gridtext::textbox_grob()}}, \code{\link[=element_markdown]{element_markdown()}}, \code{\link[=geom_textbox]{geom_textbox()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/element-markdown.R
\name{element_markdown}
\alias{element_markdown}
\title{Theme element that enables markdown text.}
\usage{
element_markdown(
  family = NULL,
  face = NULL,
  size = NULL,
  colour = NULL,
  fill = NULL,
  box.colour = NULL,
  linetype = NULL,
  linewidth = NULL,
  hjust = NULL,
  vjust = NULL,
  halign = NULL,
  valign = NULL,
  angle = NULL,
  lineheight = NULL,
  margin = NULL,
  padding = NULL,
  r = NULL,
  color = NULL,
  box.color = NULL,
  align_widths = NULL,
  align_heights = NULL,
  rotate_margins = NULL,
  debug = FALSE,
  inherit.blank = FALSE
)
}
\arguments{
\item{family}{Font family}

\item{face}{Font face}

\item{size}{Font size}

\item{colour, color}{Text color}

\item{fill}{Fill color of the enclosing box}

\item{box.colour, box.color}{Line color of the enclosing box (if different from the text color)}

\item{linetype}{Line type of the enclosing box (like \code{lty} in base R)}

\item{linewidth}{Line width of the enclosing box (measured in mm, just like \code{size} in
\code{\link[ggplot2:element]{ggplot2::element_line()}}).}

\item{hjust}{Horizontal justification}

\item{vjust}{Vertical justification}

\item{halign}{Horizontal justification}

\item{valign}{Vertical justification}

\item{angle}{Angle (in degrees)}

\item{lineheight}{Line height}

\item{padding, margin}{Padding and margins around the text box.
See \code{\link[gridtext:richtext_grob]{gridtext::richtext_grob()}} for details.}

\item{r}{Unit value specifying the corner radius of the box}

\item{align_widths, align_heights}{Should multiple elements be aligned by their
widths or height? See \code{\link[gridtext:richtext_grob]{gridtext::richtext_grob()}} for details.}

\item{rotate_margins}{Should margins get rotated in frame with rotated text?
If \code{TRUE}, the margins are applied relative to the text direction. If \code{FALSE},
the margins are applied relative to the plot direction, i.e., the top margin,
for example, is always placed above the text label, regardless of the direction
in which the text runs. The default is \code{FALSE}, which mimics the behavior of
\code{element_text()}.}

\item{debug}{Draw a debugging box around each label}

\item{inherit.blank}{See \code{\link[ggplot2:element]{ggplot2::margin()}} for details.}
}
\value{
A ggplot2 theme element that can be used inside a \code{\link[ggplot2:theme]{ggplot2::theme()}}
call.
}
\description{
Theme element that enables markdown text.
}
\seealso{
\code{\link[gridtext:richtext_grob]{gridtext::richtext_grob()}}, \code{\link[=element_textbox]{element_textbox()}}, \code{\link[=geom_richtext]{geom_richtext()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-textbox.R
\name{geom_textbox}
\alias{geom_textbox}
\title{Draw boxes containing text}
\usage{
geom_textbox(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  ...,
  nudge_x = 0,
  nudge_y = 0,
  box.padding = unit(c(5.5, 5.5, 5.5, 5.5), "pt"),
  box.margin = unit(c(0, 0, 0, 0), "pt"),
  box.r = unit(5.5, "pt"),
  width = unit(2, "inch"),
  minwidth = NULL,
  maxwidth = NULL,
  height = NULL,
  minheight = NULL,
  maxheight = NULL,
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}} or
\code{\link[ggplot2:aes_]{aes_()}}. If specified and \code{inherit.aes = TRUE} (the
default), it is combined with the default mapping at the top level of the
plot. You must supply \code{mapping} if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{stat}{The statistical transformation to use on the data for this
layer, as a string.}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function. Cannot be jointy specified with
\code{nudge_x} or \code{nudge_y}.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}

\item{nudge_x, nudge_y}{Horizontal and vertical adjustment to nudge text boxes by.
Useful for offsetting text from points, particularly on discrete scales.
Cannot be jointly specified with \code{position}.}

\item{box.padding}{Unit vector of length four specifying the padding
inside the text box.}

\item{box.margin}{Unit vector of length four specifying the margin
outside the text box.}

\item{box.r}{Unit vector of length one specifying the radius of the
box.}

\item{width, height}{Unit values specifying the width and height of
the text box (including margins!). If \code{height = NULL} (the default),
the height is chosen automatically to accommodate all the text.}

\item{minwidth, maxwidth, minheight, maxheight}{Unit values specifying
the minimum and maximum values for \code{width} and \code{height}, respectively.
If set to \code{NULL}, are not enforced.}

\item{na.rm}{If \code{FALSE}, the default, missing values are removed with
a warning. If \code{TRUE}, missing values are silently removed.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[ggplot2:borders]{borders()}}.}
}
\value{
A ggplot2 layer that can be added to a plot created with
\code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
Draw boxes of defined width and height containing word-wrapped text. Multiple
boxes can be drawn at once. Most styling parameters can be used as aesthetics
and can be applied separately to each text box drawn. The exception is styling
parameters that are specified as grid units (e.g., \code{box.padding} or \code{box.r}),
which can only be specified for all text boxes at once. See examples for details.
}
\section{Aesthetics}{


\code{geom_textbox()} understands the following aesthetics (required
aesthetics are in bold; select aesthetics are annotated):
\itemize{
\item \strong{\code{x}}
\item \strong{\code{y}}
\item \strong{\code{label}}
\item \code{alpha}
\item \code{box.colour} Color of box outline. Overrides \code{colour}.
\item \code{box.size} Width of box outline.
\item \code{colour} Default color of box text and box outline.
\item \code{family}
\item \code{fontface}
\item \code{fill} Default fill color of box background.
\item \code{group}
\item \code{halign} Horizontal alignment of text inside box.
\item \code{hjust} Horizontal alignment of box.
\item \code{lineheight}
\item \code{orientation} One of \code{"upright"}, \code{"left-rotated"},
\code{"right-rotated"}, \code{"inverted"}.
\item \code{size} Default font size of box text.
\item \code{text.colour} Color of box text. Overrides \code{colour}.
\item \code{valign} Vertical alignment of text inside box.
\item \code{vjust} Vertical alignment of box.
}
}

\examples{
library(ggplot2)

df <- data.frame(
  label = rep("Lorem ipsum dolor **sit amet,** consectetur adipiscing elit,
    sed do *eiusmod tempor incididunt* ut labore et dolore magna
    aliqua.", 2),
  x = c(0, .6),
  y = c(1, .6),
  hjust = c(0, 0),
  vjust = c(1, 0),
  orientation = c("upright", "right-rotated"),
  color = c("black", "blue"),
  fill = c("cornsilk", "white")
)

ggplot(df) +
  aes(
    x, y, label = label, color = color, fill = fill,
    hjust = hjust, vjust = vjust,
    orientation = orientation
  ) +
  geom_textbox(width = unit(0.4, "npc")) +
  geom_point(color = "black", size = 2) +
  scale_discrete_identity(aesthetics = c("color", "fill", "orientation")) +
  xlim(0, 1) + ylim(0, 1)
}
\seealso{
\code{\link[=geom_richtext]{geom_richtext()}}, \code{\link[=element_textbox]{element_textbox()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggtext.R
\docType{package}
\name{ggtext}
\alias{ggtext}
\title{Improved text rendering support for ggplot2}
\description{
The ggtext package implements both geoms (\code{\link[=geom_richtext]{geom_richtext()}}, \code{\link[=geom_textbox]{geom_textbox()}})
and theme elements (\code{\link[=element_markdown]{element_markdown()}}, \code{\link[=element_textbox]{element_textbox()}}) for improved
text rendering with ggplot2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-richtext.R, R/geom-textbox.R
\docType{data}
\name{geom_richtext}
\alias{geom_richtext}
\alias{GeomRichText}
\alias{GeomRichtext}
\alias{GeomTextBox}
\title{Richtext labels}
\usage{
geom_richtext(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  ...,
  nudge_x = 0,
  nudge_y = 0,
  label.padding = unit(c(0.25, 0.25, 0.25, 0.25), "lines"),
  label.margin = unit(c(0, 0, 0, 0), "lines"),
  label.r = unit(0.15, "lines"),
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}} or
\code{\link[ggplot2:aes_]{aes_()}}. If specified and \code{inherit.aes = TRUE} (the
default), it is combined with the default mapping at the top level of the
plot. You must supply \code{mapping} if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{stat}{The statistical transformation to use on the data for this
layer, as a string.}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function. Cannot be jointy specified with
\code{nudge_x} or \code{nudge_y}.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}

\item{nudge_x}{Horizontal and vertical adjustment to nudge labels by.
Useful for offsetting text from points, particularly on discrete scales.
Cannot be jointly specified with \code{position}.}

\item{nudge_y}{Horizontal and vertical adjustment to nudge labels by.
Useful for offsetting text from points, particularly on discrete scales.
Cannot be jointly specified with \code{position}.}

\item{label.padding}{Amount of padding around label. Defaults to 0.25 lines.}

\item{label.margin}{Unit vector of length four specifying the margin
outside the text label.}

\item{label.r}{Radius of rounded corners. Defaults to 0.15 lines.}

\item{na.rm}{If \code{FALSE}, the default, missing values are removed with
a warning. If \code{TRUE}, missing values are silently removed.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[ggplot2:borders]{borders()}}.}
}
\value{
A ggplot2 layer that can be added to a plot created with
\code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
This geom draws text labels similar to \code{\link[ggplot2:geom_text]{ggplot2::geom_label()}}, but formatted
using basic markdown/html. Parameter and aesthetic names follow the conventions
of \code{\link[ggplot2:geom_text]{ggplot2::geom_label()}}, and therefore the appearance of the frame around
the label is controlled with \code{label.colour}, \code{label.padding}, \code{label.margin},
\code{label.size}, \code{label.r}, even though the same parameters are called \code{box.colour},
\code{box.padding}, \code{box.margin}, \code{box.size}, and \code{box.r} in \code{\link[=geom_textbox]{geom_textbox()}}. Most
styling parameters can be used as aesthetics and can be applied separately to
each text label drawn. The exception is styling parameters that are specified
as grid units (e.g., \code{label.padding} or \code{label.r}), which can only be specified
for all text labels at once. See examples for details.
}
\section{Aesthetics}{


\code{geom_richtext()} understands the following aesthetics (required
aesthetics are in bold; select aesthetics are annotated):
\itemize{
\item \strong{\code{x}}
\item \strong{\code{y}}
\item \strong{\code{label}}
\item \code{alpha}
\item \code{angle}
\item \code{colour} Default color of label text and label outline.
\item \code{family}
\item \code{fontface}
\item \code{fill} Default fill color of label background.
\item \code{group}
\item \code{hjust}
\item \code{label.colour} Color of label outline. Overrides \code{colour}.
\item \code{label.size} Width of label outline.
\item \code{lineheight}
\item \code{size} Default font size of label text.
\item \code{text.colour} Color of label text. Overrides \code{colour}.
\item \code{vjust}
}
}

\examples{
library(ggplot2)

df <- data.frame(
  label = c(
    "Some text **in bold.**",
    "Linebreaks<br>Linebreaks<br>Linebreaks",
    "*x*<sup>2</sup> + 5*x* + *C*<sub>*i*</sub>",
    "Some <span style='color:blue'>blue text **in bold.**</span><br>And *italics text.*<br>
      And some <span style='font-size:18pt; color:black'>large</span> text."
  ),
  x = c(.2, .1, .5, .9),
  y = c(.8, .4, .1, .5),
  hjust = c(0.5, 0, 0, 1),
  vjust = c(0.5, 1, 0, 0.5),
  angle = c(0, 0, 45, -45),
  color = c("black", "blue", "black", "red"),
  fill = c("cornsilk", "white", "lightblue1", "white")
)

ggplot(df) +
  aes(
    x, y, label = label, angle = angle, color = color, fill = fill,
    hjust = hjust, vjust = vjust
  ) +
  geom_richtext() +
  geom_point(color = "black", size = 2) +
  scale_color_identity() +
  scale_fill_identity() +
  xlim(0, 1) + ylim(0, 1)
  
# labels without frame or background are also possible
ggplot(df) +
  aes(
    x, y, label = label, angle = angle, color = color,
    hjust = hjust, vjust = vjust
  ) +
  geom_richtext(
    fill = NA, label.color = NA, # remove background and outline
    label.padding = grid::unit(rep(0, 4), "pt") # remove padding
  ) +
  geom_point(color = "black", size = 2) +
  scale_color_identity() +
  xlim(0, 1) + ylim(0, 1)
}
\seealso{
\code{\link[=geom_textbox]{geom_textbox()}}, \code{\link[=element_markdown]{element_markdown()}}
}
\keyword{datasets}
